/*
 *  ========================================================================
 *  DIScrete event baSed Energy Consumption simulaTor 
 *    					             for Clouds and Federations (DISSECT-CF)
 *  ========================================================================
 *  
 *  This file is part of DISSECT-CF.
 *  
 *  DISSECT-CF is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *  
 *  DISSECT-CF is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 *  General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with DISSECT-CF.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  (C) Copyright 2013, Gabor Kecskemeti (gkecskem@dps.uibk.ac.at,
 *   									  kecskemeti.gabor@sztaki.mta.hu)
 */
package hu.mta.sztaki.lpds.cloud.simulator.energy;

import java.util.LinkedList;

public class MonitorConsumption {
	private class SpreadingRecord {
		public long timestamp;
		public long totalTransferred;
	}

	private LinkedList<SpreadingRecord> subSecondRecords = new LinkedList<SpreadingRecord>();
	private LinkedList<SpreadingRecord> subHourRecords = new LinkedList<SpreadingRecord>();
	private LinkedList<SpreadingRecord> subDayRecords = new LinkedList<SpreadingRecord>();
	private long subSecondTransfers = 0;
	private long subHourTransfers = 0;
	private long subDayTransfers = 0;
	private long totalTransferred = 0;
	
	private void updateRecords(final long timeStamp) {
		final SpreadingRecord current = new SpreadingRecord();
		current.timestamp = timeStamp;
		current.totalTransferred = totalTransferred;
		subSecondRecords.add(current);
		while (subSecondRecords.peek().timestamp + 1000 < timeStamp) {
			subHourRecords.add(subSecondRecords.poll());
			while (subHourRecords.peek().timestamp + 3600000 < timeStamp) {
				subDayRecords.add(subHourRecords.poll());
				while (subDayRecords.peek().timestamp + 86400000 < timeStamp) {
					subDayRecords.poll();
				}
			}
		}
		subSecondTransfers = totalTransferred
				- subSecondRecords.peek().totalTransferred;
		try {
			subHourTransfers = totalTransferred
					- subHourRecords.peek().totalTransferred;
		} catch (NullPointerException ne) {
			subHourTransfers = subSecondTransfers;
		}
		try {
			subDayTransfers = totalTransferred
					- subDayRecords.peek().totalTransferred;
		} catch (NullPointerException ne) {
			subDayTransfers = subHourTransfers;
		}
	}

	public long getTotalTransferred() {
		return totalTransferred;
	}

	public long getSubDayTransfers() {
		return subDayTransfers;
	}

	public long getSubHourTransfers() {
		return subHourTransfers;
	}

	public long getSubSecondTransfers() {
		return subSecondTransfers;
	}
}
