/*
 *  ========================================================================
 *  DIScrete event baSed Energy Consumption simulaTor 
 *    					             for Clouds and Federations (DISSECT-CF)
 *  ========================================================================
 *  
 *  This file is part of DISSECT-CF.
 *  
 *  DISSECT-CF is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *  
 *  DISSECT-CF is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 *  General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with DISSECT-CF.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  (C) Copyright 2013, Gabor Kecskemeti (gkecskem@dps.uibk.ac.at,
 *   									  kecskemeti.gabor@sztaki.mta.hu)
 */
package hu.mta.sztaki.lpds.cloud.simulator.iaas.resourcemodel;

import java.util.Iterator;

import hu.mta.sztaki.lpds.cloud.simulator.Timed;

public class OfferResources extends ResourceSpreader {

	public OfferResources(final double initialProcessingPower) {
		super(initialProcessingPower);
	}

	@Override
	public int compareTo(Timed o) {
		final int returner = super.compareTo(o);
		return ((returner == 0) && (o instanceof UtilizeResources)) ? -1
				: returner;
	}

	@Override
	protected double singleSpread(double toSpread) {
		double remainingBandwidth = toSpread * toProcess.size();
		final Iterator<ResourceConsumption> i = toProcess.iterator();
		while (i.hasNext()) {
			final ResourceConsumption con = i.next();
			if (con.toBeProcessed > 0) {
				final double realRequestedContent = con.underProcessing
						+ (toSpread < con.toBeProcessed ? toSpread
								: con.toBeProcessed);
				final double cc = realRequestedContent < timeAdjustedProcessingPower ? realRequestedContent
						: timeAdjustedProcessingPower;
				final double transferdiff = cc - con.underProcessing;
				con.underProcessing = cc;
				con.toBeProcessed -= transferdiff;

				if (con.toBeProcessed <= 0) {
					i.remove();
				}

				remainingBandwidth -= transferdiff;
			}
		}
		return remainingBandwidth;
	}

	@Override
	protected ResourceSpreader getCounterPart(ResourceConsumption con) {
		return con.getConsumer();
	}

	@Override
	public String toString() {
		return "Offer(" + toProcess + ")";
	}
}
