/*
 *  ========================================================================
 *  DIScrete event baSed Energy Consumption simulaTor 
 *    					             for Clouds and Federations (DISSECT-CF)
 *  ========================================================================
 *  
 *  This file is part of DISSECT-CF.
 *  
 *  DISSECT-CF is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *  
 *  DISSECT-CF is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 *  General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with DISSECT-CF.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  (C) Copyright 2013, Gabor Kecskemeti (gkecskem@dps.uibk.ac.at,
 *   									  kecskemeti.gabor@sztaki.mta.hu)
 */
package hu.mta.sztaki.lpds.cloud.simulator.iaas.resourcemodel;

public class ResourceConsumption {

	public static interface ConsumptionEvent {
		public void conComplete();


		public void conCancelled(ResourceConsumption problematic);
	}

	double underProcessing;
	double toBeProcessed;
	final ConsumptionEvent ev;

	private ResourceSpreader consumer;
	private ResourceSpreader provider;

	private boolean resumable = true;
	private boolean registered = false;

	public ResourceConsumption(final double total, final ResourceSpreader in,
			final ResourceSpreader out, final ConsumptionEvent e) {
		underProcessing = 0;
		toBeProcessed = total;
		ev = e;
		this.consumer = in;
		this.provider = out;
	}

	public void registerConsumption() {
		if (resumable) {
			provider.registerConsumption(this);
			consumer.registerConsumption(this);
			registered = true;
		}
	}

	public double getUnProcessed() {
		return underProcessing + toBeProcessed;
	}

	public void cancel() {
		suspend();
		resumable = false;
		ev.conCancelled(this);
	}

	public void suspend() {
		provider.cancelConsumption(this);
		consumer.cancelConsumption(this);
		registered = false;
	}

	public void setProvider(final ResourceSpreader provider) {
		if (!registered) {
			this.provider = provider;
			return;
		} else {
			throw new IllegalStateException(
					"Attempted consumer change with a registered consumption");
		}
	}

	public void setConsumer(final ResourceSpreader consumer) {
		if (!registered) {
			this.consumer = consumer;
			return;
		} else {
			throw new IllegalStateException(
					"Attempted consumer change with a registered consumption");
		}
	}

	public ResourceSpreader getConsumer() {
		return consumer;
	}

	public ResourceSpreader getProvider() {
		return provider;
	}

	@Override
	public String toString() {
		return "RC(C:" + underProcessing + " T:" + toBeProcessed + ")";
	}
}
