/*
 *  ========================================================================
 *  DIScrete event baSed Energy Consumption simulaTor 
 *    					             for Clouds and Federations (DISSECT-CF)
 *  ========================================================================
 *  
 *  This file is part of DISSECT-CF.
 *  
 *  DISSECT-CF is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *  
 *  DISSECT-CF is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 *  General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with DISSECT-CF.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  (C) Copyright 2013, Gabor Kecskemeti (gkecskem@dps.uibk.ac.at,
 *   									  kecskemeti.gabor@sztaki.mta.hu)
 */
package hu.mta.sztaki.lpds.cloud.simulator.iaas.resourcemodel;

import hu.mta.sztaki.lpds.cloud.simulator.Timed;

import java.util.Comparator;
import java.util.PriorityQueue;

public abstract class ResourceSpreader extends Timed {
	static final Comparator<ResourceConsumption> consComp = new Comparator<ResourceConsumption>() {
		@Override
		public int compare(final ResourceConsumption o1,
				final ResourceConsumption o2) {
			final double upOth = o1.getUnProcessed();
			final double upThis = o2.getUnProcessed();
			return upOth < upThis ? -1 : (upOth == upThis ? 0 : 1);
		}

		@Override
		public boolean equals(Object obj) {
			System.err.println("WHOO!");
			System.exit(0);
			return false;
		}
	};
	protected double perSecondProcessingPower;
	protected final PriorityQueue<ResourceConsumption> toProcess = new PriorityQueue<ResourceConsumption>(
			500, consComp);

	protected long lastNotifTime = 0;
	protected double timeAdjustedProcessingPower;
	private double totalProcessed = 0;
	protected int nonReceivers;

	public ResourceSpreader(final double initialProcessingPower) {
		perSecondProcessingPower = initialProcessingPower;
	}

	protected long calcNewFreq() {
		final int prSize = toProcess.size();
		if (prSize > 0) {
			final double minimum = toProcess.peek().getUnProcessed();
			return 1 + (long) (minimum * 1000 / ((double) perSecondProcessingPower / prSize));
		}
		return 1;
	}

	protected void updateForAllCons(long newfreq) {
		frequpdate: while (true) {
			final long oldNe = getNextEvent();
			final long neOur = updateFrequency(newfreq);
			if (oldNe == neOur) {
				return;
			}
			for (final ResourceConsumption con : toProcess) {
				final ResourceSpreader counterPart = getCounterPart(con);
				if (counterPart.isSubscribed()) {
					final long neCP = counterPart.getNextEvent();
					if (neOur < neCP) {
						counterPart.updateForAllCons(newfreq);
					} else if (neOur != neCP) {
						final long nfupd = counterPart.nextEventDistance();
						if (nfupd != 0) {
							newfreq = nfupd;
							continue frequpdate;
						}
					}
				}
			}
			return;
		}
	}

	public void registerConsumption(final ResourceConsumption con) {
		toProcess.offer(con);
		final long nf = calcNewFreq();
		if (isSubscribed()) {
			updateForAllCons(nf);
			return;
		} else {
			lastNotifTime = Timed.getFireCount();
			subscribe(nf);
		}
	}

	public void cancelConsumption(final ResourceConsumption con) {
		final boolean remresult = toProcess.remove(con);
		if (remresult) {
			if (toProcess.size() > 0) {
				updateForAllCons(calcNewFreq());
			} else {
				unsubscribe();
				final ResourceSpreader cp = getCounterPart(con);
				cp.updateForAllCons(cp.calcNewFreq());
			}
		}
	}

	@Override
	public void tick(final long currentFireCount) {
		final double elapsedFires = currentFireCount - lastNotifTime;
		double remainingProcessingPower = timeAdjustedProcessingPower = perSecondProcessingPower
				* (elapsedFires / 1000);
		double pastRemaining;
		nonReceivers = 0;
		try {
			do {
				pastRemaining = remainingProcessingPower;
				remainingProcessingPower = singleSpread(pastRemaining
						/ (toProcess.size() - nonReceivers));
			} while (remainingProcessingPower > 0
					&& pastRemaining != remainingProcessingPower);
		} catch (ArithmeticException e) {
		}
		totalProcessed += timeAdjustedProcessingPower
				- remainingProcessingPower;

		if (toProcess.size() == 0) {
			unsubscribe();
			return;
		} else {
			updateForAllCons(calcNewFreq());
			lastNotifTime = currentFireCount;
		}
	}

	public double getTotalProcessed() {
		return totalProcessed;
	}

	protected abstract double singleSpread(final double toSpread);

	protected abstract ResourceSpreader getCounterPart(
			final ResourceConsumption con);

	public double getPerSecondProcessingPower() {
		return perSecondProcessingPower;
	}

	protected void setPerSecondProcessingPower(double perSecondProcessingPower) {
		this.perSecondProcessingPower = perSecondProcessingPower;
		if (isSubscribed()) {
			updateForAllCons(calcNewFreq());
		}
	}
}
