/*
 *  ========================================================================
 *  DIScrete event baSed Energy Consumption simulaTor 
 *    					             for Clouds and Federations (DISSECT-CF)
 *  ========================================================================
 *  
 *  This file is part of DISSECT-CF.
 *  
 *  DISSECT-CF is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *  
 *  DISSECT-CF is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 *  General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with DISSECT-CF.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  (C) Copyright 2013, Gabor Kecskemeti (gkecskem@dps.uibk.ac.at,
 *   									  kecskemeti.gabor@sztaki.mta.hu)
 */

package hu.mta.sztaki.lpds.cloud.simulator.util;

import java.util.Arrays;
import java.util.List;
import java.util.Vector;

public class FastArray<E> {

	public static interface IteratingAction<E> {
		public boolean evalItem(E element, int index);
	}

	public static interface ForEachAction<E> {
		public void eval(E element);
	}

	public static interface ForEachWithFeedback<E> {
		public boolean eval(E element);
	}

	public interface WithIndexAction<E> {
		public void eval(E element, int index);
	}

	protected E[] array;
	protected int lastitem = 0;

	public FastArray(final E[] origarr) {
		array = origarr;
		Arrays.fill(array, null);
	}

	public int add(final E el) {
		int liextra = lastitem + 1;
		if (liextra == array.length) {
			array = Arrays.copyOf(array, 2 * lastitem);
		}
		array[lastitem] = el;
		lastitem = liextra;
		return lastitem;
	}
	
	public int insert(final E el, int index) {
		if(lastitem<index) {
			return add(el);
		}
		int liextra = lastitem+1;
		if(liextra == array.length) {
			array = Arrays.copyOf(array, 2*lastitem);
		}
		for(int i=lastitem - 1; index<=i;i--) {
			array[i+1]=array[i];
		}
		array[index]=el;
		lastitem = liextra;
		return lastitem;
	}

	public int remove(final E el) {
		for (int i = lastitem - 1; 0 <= i; i--) {
			if (array[i] == el) {
				return remove(i);
			}
		}
		return -1;
	}

	public int remove(int i) {
		while (i < lastitem) {
			array[i] = array[++i];
		}
		return --lastitem;
	}

	public int size() {
		return lastitem;
	}

	public int iterate(final IteratingAction<E> action, final int startindex) {
		for (int i = lastitem - 1; startindex <= i; i--) {
			if (action.evalItem(array[i], i)) {
				return i;
			}
		}
		return -1;
	}

	public int iterate(final ForEachWithFeedback<E> action, final int startindex) {
		for (int i = lastitem - 1; startindex <= i; i--) {
			if (action.eval(array[i])) {
				return i;
			}
		}
		return -1;
	}

	public void iterate(final ForEachAction<E> action) {
		for (int i = lastitem - 1; 0 <= i; i--) {
			action.eval(array[i]);
		}
	}

	public void iterate(final WithIndexAction<E> action) {
		for (int i = lastitem - 1; 0 <= i; i--) {
			action.eval(array[i], i);
		}
	}

	public List<E> asList() {
		Vector<E> list = new Vector<E>(lastitem);
		for (int i = lastitem - 1; 0 <= i; i--) {
			list.add(array[i]);
		}
		return list;
	}

	public E[] getManagedArray() {
		return array;
	}

	public E get(final int index) {
		return array[index];
	}
}
